<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Country;
use App\Models\Order;
use App\Models\Token;
use App\Models\User;
use App\Repositories\CategoryRepository;
use App\Repositories\CountriesRepository;
use App\Repositories\usersRepository;
use App\Repositories\UserRepository;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;


class UsersController extends Controller
{
    private $usersRepository;


    public function __construct(UserRepository $usersRepository)
    {
        $this->usersRepository = $usersRepository;
    }


    public function index($type)
    {
        $rows = $this->usersRepository->index($type);
        return view('admin.users.index', compact('rows','type'));
    }
    public function orders($id)
    {
        $rows =User::findOrFail($id)->orders;
        return view('admin.orders.index', compact('rows'));
    }


    public function edit($id)
    {
        $row = $this->usersRepository->show($id);

        return view('admin.users.edit', compact('row',));
    }

    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:users,email,' . $id,
            'status' => 'required|in:active,not_active',
            'mobile' => 'required|unique:users,mobile,' . $id,
            'poet_types' => 'required_if:type,poet|required_if:type,class_manager|in:dialogue,elite,all',
            'image' => 'image'
        ]);
        $this->usersRepository->update($id,$request);
        success_toast();
        $user=User::find($id);
        return redirect()->route('users.index',[$user->type]);
    }

    public function create($type){
        return view('admin.users.create',compact('type'));
    }
    public function store(Request $request)
    {

        $this->validate($request, [
            'name' => 'required',
            'type' => 'required|in:client,poet,class_manager',
            'email' => 'required|email|unique:users,email',
            'city_id' => 'required|exists:cities,id',
            'poet_types' => 'required_if:type,poet|required_if:type,class_manager|in:dialogue,elite,all',
            'status' => 'required|in:active,not_active',
            'mobile' => 'required|unique:users,mobile',
            'image' => 'image'
        ]);
        $this->usersRepository->store($request);
        toast('تم الإنشاء بنجاح', 'success');
        return redirect()->route('users.index',[$request->type]);
    }

    public function destroy($id)
    {
        $this->usersRepository->destroy($id);
        success_toast();
        return back();
    }



}
