<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\AdResource;
use App\Http\Resources\CityResource;
use App\Http\Resources\UserResource;
use App\Models\Ad;
use App\Models\City;
use App\Models\ContactUs;
use App\Models\Setting;
use App\Models\User;
use Illuminate\Http\Request;

class DataController extends Controller
{
    public function ads()
    {
        $ads = Ad::latest()->get();
        return AdResource::collection($ads)->additional(['value' => true, 'code' => 200]);
    }

    public function contact_us(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'name' => 'required',
            'mobile' => 'required',
            'note' => 'required'
        ]);
        if ($validator->passes()) {
            $contact = new ContactUs();
            $contact['name'] = $request['name'];
            $contact['mobile'] = $request['mobile'];
            $contact['note'] = $request['note'];
            $contact->save();
            return response()->json(['value' => true, 'data' => 'تم ارسال الرسالة بنجاح', 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function cities()
    {
        $rows = City::all();
        return CityResource::collection($rows)->additional([
            'value' => true, 'code' => 200
        ]);
    }

    public function settings(Request $request)
    {
        $setting = Setting::first();
        $data['bank_name'] = ($setting && $setting->bank_name) ? $setting->bank_name : '';
        $data['bank_number'] = ($setting && $setting->bank_number) ? $setting->bank_number : '';
        $data['bank_iban'] = ($setting && $setting->bank_iban) ? $setting->bank_iban : '';
        $data['policy'] = ($setting && $setting->policy) ? $setting->policy : '';
        $data['email'] = ($setting && $setting->email) ? $setting->email : '';
        $data['mobile'] = ($setting && $setting->mobile) ? $setting->mobile : '';
        $data['about_us'] = ($setting && $setting->about_us) ? $setting->about_us : '';
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
    }


    public function search(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'city_id' => 'nullable|exists:cities,id'
        ]);
        if ($validator->passes()) {

            $rows = User::whereIn('type', ['class_manager', 'poet'])->where('status', 'active');
            if ($request->search) {
                $rows->where('name', 'LIKE', "%{$request->search}%");
            }
            if ($request->city_id) {
                $rows->where('city_id', $request->city_id);
            }
            $data = $rows->paginate(10);
            return UserResource::collection($data)->additional(['value' => true, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }


    public function send_sms(Request  $request){
        shamel_sms('066550207398',$request->message);
        return 'yes';
    }



}
