<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\OrderResource;
use App\Models\Notification;
use App\Models\Order;
use App\Models\PaymentTransfer;
use App\Models\Setting;
use App\Models\Token;
use Carbon\Carbon;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use JWTAuth;
use Alkoumi\LaravelHijriDate\Hijri;

class OrdersController extends Controller
{
    public function transfers(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'image' => 'required|image',
        ]);
        if ($validator->passes()) {
            $transfer = new PaymentTransfer();
            $transfer->user_id = $user->id;
            if ($request->hasFile('image')) {
                $transfer->setImageAttribute();
            }
            $transfer->save();
            return response()->json(['value' => true, 'data' => 'تم ارسال الايصال بنجاح', 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function orders($type)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        if ($user->type == 'client') {
            if ($type == 'waiting') {
                $rows = Order::whereIn('type', ['waiting', 'accept'])->where('client_id', $user->id)->latest()->paginate(10);
                $order_number = Order::whereIn('type', ['waiting', 'accept'])->where('client_id', $user->id)->count();
            } else {
                $rows = Order::where('type', $type)->where('client_id', $user->id)->latest()->paginate(10);
                $order_number = Order::where('type', $type)->where('client_id', $user->id)->count();
            }
        } else {
            if ($type == 'waiting') {
                $rows = Order::whereIn('type', ['waiting', 'accept'])->where('user_id', $user->id)->latest()->paginate(10);
                $order_number = Order::whereIn('type', ['waiting', 'accept'])->where('user_id', $user->id)->count();
            } else {
                $rows = Order::where('type', $type)->where('user_id', $user->id)->latest()->paginate(10);
                $order_number = Order::where('type', $type)->where('user_id', $user->id)->count();
            }
        }
        return OrderResource::collection($rows)->additional(['order_number' => $order_number, 'value' => true, 'code' => 200]);
    }


    public function single_order($id)
    {
//        $string = '17-1-1443';
//       $arr = explode('-', $string);
//        $r = HijriToJD($arr[1], $arr[0], $arr[2]);
//
//        $c = Carbon::parse(jdtogregorian($r) . '10:30:00');
//
//        dd(jdtogregorian($r), Hijri::FullDateIndicDigits($c));
//        dd(jdtogregorian($r), Hijri::DateIndicDigits('l ، j F ، Y', $c));


        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }

        $row = Order::find($id);
        $data = new OrderResource($row);
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
    }

    public function send_client_request_order(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'date' => 'required',
            'time' => 'required',
            'user_id' => 'required|exists:users,id'
        ]);
        if ($validator->passes()) {
            $setting = Setting::first();

            if ($setting && $setting->limit_price && $user->wallet <= '-' . $setting->limit_price) {
                return response()->json(['value' => true, 'data' => 'برجاء تسديد القيمه المستحقة فى المحفظه اولا ', 'code' => 200]);
            }
            $date = convert_date($request->date, $request->time);

            $order = Order::where(['user_id' => $request->user_id, 'type' => 'in_progress'])->whereDate('date', '=', Carbon::parse($date)->toDateString())->first();

            if ($order || Carbon::parse($date)->toDateString() == Carbon::now()->toDateString()) {
                return response()->json(['value' => true, 'data' => 'غير متاح فى هذا اليوم', 'code' => 200]);
            }
            $new_order = new Order();
            $new_order->client_id = $user->id;
            $new_order->user_id = $request->user_id;
            $new_order->date = $date;
            $new_order->type = 'waiting';
            $new_order->city_id = $user->city->id;
            $new_order->save();


            $notification = new Notification();
            $notification['title'] = "طلب جديد";
            $notification['body'] = "تم ارسال طلب جديد تحت المراجعه من $user->name";
            $notification['type'] = 'order';
            $notification['user_id'] = $request->user_id;
            $notification['order_id'] = $new_order->id;
            $notification->save();

            $tokens = Token::where('user_id', $request->user_id)->get();
            foreach ($tokens as $token) {

                $push = new PushNotification('fcm');
                if ($token->device_type == 'ios') {

                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'notification' => [
                            'title' => $notification->title,
                            'body' => $notification->body,
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                } else {
                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                }
            }

            return response()->json(['value' => true, 'data' => 'تم الارسال وطلبك تحت المرجعه الان', 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function accept_cancel_request_order(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'status' => 'required|in:accept,cancel',
        ]);
        if ($validator->passes()) {

            $order = Order::where(['id' => $request->order_id, 'user_id' => $user->id])->first();
            if (!$order) {
                return response()->json(['value' => true, 'data' => "ليس لديك الصلاحيات", 'code' => 200]);
            }
            $order->type = $request->status;
            $order->update();

            if ($request->status == 'accept') {
                $message = "تم قبول طلبكم من $user->name يمكنك الان التحدث معه ";
                $data = 'تم قبول الطلب';
            } else {

                $message = "تم رفض طلبكم من $user->name وتم الغاء الطلب ";
                $data = 'تم رفض الطلب';
            }

            $notification = new Notification();
            $notification['title'] = $data;
            $notification['body'] = $message;
            $notification['type'] = 'order';
            $notification['user_id'] = $order->client->id;
            $notification['order_id'] = $order->id;
            $notification->save();

            $tokens = Token::where('user_id', $request->user_id)->get();
            foreach ($tokens as $token) {

                $push = new PushNotification('fcm');
                if ($token->device_type == 'ios') {

                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'notification' => [
                            'title' => $notification->title,
                            'body' => $notification->body,
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                } else {
                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                }
            }
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function cancel_client_order(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id'
        ]);
        if ($validator->passes()) {

            $setting = Setting::first();

            $order = Order::where(['id' => $request->order_id, 'client_id' => $user->id, 'type' => 'in_progress'])->first();
            if (!$order) {
                return response()->json(['value' => true, 'data' => "ليس لديك الصلاحيات", 'code' => 200]);
            }
            if (Carbon::now() <= Carbon::parse($order->date)->subDay()) {

                $order->type = 'cancel';
                $order->update();
                //commission_manager
                //commission_poet
                if ($order->user->type == 'poet') {
                    $user_commission = $setting && $setting->commission_poet ? $setting->commission_poet : 0;
                } elseif ($order->user->type == 'class_manager') {
                    $user_commission = $setting && $setting->commission_manager ? $setting->commission_manager : 0;
                }
                $client_price = $order->price * $setting->commission_client / 100;
                $user_price = $order->price * $user_commission / 100;
                $user->wallet += $client_price;
                $user->update();

                $user_order = $order->user;
                $user_order->wallet += $user_price;
                $user_order->update();

                $notification = new Notification();
                $notification['title'] = " تم الغاء الطلب";
                $notification['body'] = "$order->id تم الغاء الطلب رقم  ";
                $notification['type'] = 'order';
                $notification['user_id'] = $order->user->id;
                $notification['order_id'] = $order->id;
                $notification->save();

                $tokens = Token::where('user_id', $order->user_id)->get();
                foreach ($tokens as $token) {

                    $push = new PushNotification('fcm');
                    if ($token->device_type == 'ios') {

                        $msg = [
                            'data' => [
                                'title' => $notification->title,
                                'sound' => 'default',
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                                'body' => $notification->body,
                                'status' => 'order',
                                'order_id' => $notification->order_id

                            ],
                            'notification' => [
                                'title' => $notification->title,
                                'body' => $notification->body,
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();
                    } else {
                        $msg = [
                            'data' => [
                                'title' => $notification->title,
                                'sound' => 'default',
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                                'body' => $notification->body,
                                'status' => 'order',
                                'order_id' => $notification->order_id

                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();
                    }
                }
                return response()->json(['value' => true, 'data' => 'تم الغاء الطلب بنجاح', 'code' => 200]);
            }

            return response()->json(['value' => true, 'data' => 'لا يمكن الغاء الطلب', 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function send_offer_to_client(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'price' => 'required'
        ]);
        if ($validator->passes()) {

            $order = Order::where(['id' => $request->order_id, 'user_id' => $user->id, 'type' => 'accept'])->first();
            if (!$order) {
                return response()->json(['value' => true, 'data' => "ليس لديك الصلاحيات", 'code' => 200]);
            }

            if ($order->price) {
                return response()->json(['value' => true, 'data' => "تم ارسال لعرض مسبقا", 'code' => 200]);
            }
            $order->price = $request->price;
            $order->update();


            $notification = new Notification();
            $notification['title'] = " تم ارسال عرض سعر";
            $notification['body'] = "$order->id تم ارسال عرض سعر على الطلب رقم  ";
            $notification['type'] = 'order';
            $notification['user_id'] = $order->client->id;
            $notification['order_id'] = $order->id;
            $notification->save();

            $tokens = Token::where('user_id', $order->client_id)->get();
            foreach ($tokens as $token) {

                $push = new PushNotification('fcm');
                if ($token->device_type == 'ios') {

                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'notification' => [
                            'title' => $notification->title,
                            'body' => $notification->body,
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                } else {
                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                }
            }
            return response()->json(['value' => true, 'data' => 'تم ارسال عرض السعر بنجاح', 'code' => 200]);


        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function accept_cancel_offer(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'status' => 'required|in:in_progress,cancel'
        ]);
        if ($validator->passes()) {
            $setting = Setting::first();

            $order = Order::where(['id' => $request->order_id, 'client_id' => $user->id, 'type' => 'accept'])->first();
            if (!$order) {
                return response()->json(['value' => true, 'data' => "ليس لديك الصلاحيات", 'code' => 200]);
            }
            $order->type = $request->status;
            $order->update();


            if ($request->status == 'in_progress') {

                //commission_manager
                //commission_poet
                if ($order->user->type == 'poet') {
                    $user_commission = $setting && $setting->commission_poet ? $setting->commission_poet : 0;
                } elseif ($order->user->type == 'class_manager') {
                    $user_commission = $setting && $setting->commission_manager ? $setting->commission_manager : 0;
                }

                $client_price = $order->price * $setting->commission_client / 100;
                $user->wallet -= $client_price;
                $user->update();

                $user_price = $order->price * $user_commission / 100;


                $user_order = $order->user;
                $user_order->wallet -= $user_price;
                $user_order->update();


                $message = " $order->id تم الموافقه على عرض السعر الخاص بطلب رقم";
                $data = 'تم الموافقه على عرض السعر';
            } else {

                $message = "$order->id تم رفض عرض السعر والغاء الطلب رقم ";
                $data = 'تم رفض عرض السعر والغاء الطلب';
            }


            $notification = new Notification();
            $notification['title'] = $data;
            $notification['body'] = $message;
            $notification['type'] = 'order';
            $notification['user_id'] = $order->user->id;
            $notification['order_id'] = $order->id;
            $notification->save();

            $tokens = Token::where('user_id', $order->user_id)->get();
            foreach ($tokens as $token) {

                $push = new PushNotification('fcm');
                if ($token->device_type == 'ios') {

                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'notification' => [
                            'title' => $notification->title,
                            'body' => $notification->body,
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                } else {
                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                }
            }
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);


        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function client_finish_order(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id'
        ]);
        if ($validator->passes()) {

            $order = Order::where(['id' => $request->order_id, 'client_id' => $user->id, 'type' => 'in_progress'])->first();
            if (!$order) {
                return response()->json(['value' => true, 'data' => "ليس لديك الصلاحيات", 'code' => 200]);
            }

            $order->type = "finished";
            $order->update();


            $notification = new Notification();
            $notification['title'] = " تم انتهاء الطلب";
            $notification['body'] = "$order->id تم انتهاء الطلب رقم ";
            $notification['type'] = 'order';
            $notification['user_id'] = $order->user->id;
            $notification['order_id'] = $order->id;
            $notification->save();

            $tokens = Token::where('user_id', $order->user_id)->get();
            foreach ($tokens as $token) {

                $push = new PushNotification('fcm');
                if ($token->device_type == 'ios') {

                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'notification' => [
                            'title' => $notification->title,
                            'body' => $notification->body,
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                } else {
                    $msg = [
                        'data' => [
                            'title' => $notification->title,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            'body' => $notification->body,
                            'status' => 'order',
                            'order_id' => $notification->order_id

                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();
                }
            }
            return response()->json(['value' => true, 'data' => 'تم انهاء الطلب', 'code' => 200]);


        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }


}
