<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\UserResource;
use App\Models\Token;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Exceptions\JWTException;
use JWTAuth;

class UsersController extends Controller
{
    public function register(Request $request)
    {

        $validator = \Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|email|unique:users,email',
            'mobile' => ['required', 'unique:users,mobile'],
            'password' => 'required|confirmed|min:6',
            'password_confirmation' => 'sometimes|required_with:password',
            'city_id' => 'required|exists:cities,id',
            'type' => 'required|in:client,poet,class_manager',
            'poet_types' => 'required_if:type,poet|in:dialogue,elite,all|nullable',
            'note' => 'required_if:type,poet,class_manager',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android'

        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['name'] = $request['name'];
            $user['email'] = $request['email'];
            $user['mobile'] = $request['mobile'];
            $user['password'] = bcrypt($request['password']);
            $user['type'] = $request['type'];
            $user['city_id'] = $request['city_id'];
            $user['poet_types'] = $request['poet_types'];
            $user['note'] = $request['note'];
            $user['verification_code'] = rand(1111, 9999);
            $user['status'] = 'not_active';
            $user->save();
            // SEND SMS VERIFICATION CODE

            shamel_sms($user->mobile, "Welcome in Khaloha App , Your Code is  : $user->verification_code");


            $check = Token::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                $check->user_id = $user->id;
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                Token::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function login(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required',
            'password' => 'required',
            'type' => 'required|in:client,poet,class_manager',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',

        ]);
        if ($validator->passes()) {

//            $auth = ['mobile' => $request->input('mobile'), 'password' => $request->input('password'), 'type' => $request->input('type')];

            try {
                if (!$token = JWTAuth::attempt(['mobile' => $request->input('mobile'), 'password' => $request->input('password'), 'type' => $request->input('type')], ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                    return response()->json(['value' => false, 'msg' => 'رقم الجوال او الرقم السري خطأ'], 200);
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }
            $user = User::whereMobile($request['mobile'])->first();

            $check = Token::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;

                $check->update();
            } else {
                Token::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function forget_password(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required'
        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => 'رقم الجوال غير مسجل', 'code' => 422]);
            $code = rand(1111, 9999);
            $user->verification_code = $code;
            $user->update();

            // code sms
            shamel_sms($user->mobile, " كود التفعيل هوا : $user->verification_code");

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function check_forget_code(Request $request)
    {

        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'code' => 'required|exists:users,verification_code'
        ]);
        if ($validator->passes()) {
            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => 'رقم الجوال غير مسجل', 'code' => 422]);
            if ($user->verification_code != $request['code']) return response()->json(['value' => false, 'msg' => 'الرمز غير صحبح', 'code' => 422]);

//            $user->status = 'active';
            $user->verification_code = null;
            $user->update();
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'msg' => 'الرمز صحيح', 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_password(Request $request)
    {

        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'password' => 'required|min:6',
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios',

        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            $user->password = bcrypt($request['password']);
            $user->update();
            $check = Token::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;

                $check->update();
            } else {
                Token::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                ]);
            }
            $data = new UserResource($user);

            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function logout(Request $request)
    {

        $validator = \Validator::make($request->all(), [
            'device_token' => 'required|exists:tokens,token'
        ]);
        if ($validator->passes()) {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
            }


            $exist_token = Token::where(['token' => $request['device_token']])->first();
            if (!$exist_token) return response()->json(['value' => false, 'msg' => 'device token and auth not match', 'code' => 401]);

            $exist_token->status = 'disconnected';
            $exist_token->update();

            JWTAuth::invalidate(JWTAuth::getToken());
            return response()->json(['value' => true, 'data' => 'تم تسجيل الخروج بنجاح', 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }

    public function change_old_password(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'old_password' => 'required',
            'new_password' => 'required|confirmed',
            'new_password_confirmation' => 'required'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
            }

            if (\Hash::check($request['old_password'], $user->password)) {
                $user->password = bcrypt($request['new_password']);
                $user->update();
                $data = new UserResource($user);
                return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => 'كلمة المرور الحالية غير صحيحه', 'code' => 422]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function active_account(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'code' => 'required|exists:users,verification_code'
        ]);
        if ($validator->passes()) {

            if ($user->verification_code != $request['code']) return response()->json(['value' => false, 'msg' => 'الرمز غير صحبح', 'code' => 422]);

            $user->status = 'active';
            $user->verification_code = null;
            $user->update();
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'msg' => 'تم تفعيل الحساب', 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function resend_code(Request $request)
    {

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }

        $user['verification_code'] = rand(1111, 9999);
        $user->update();
        //send code
        shamel_sms($user->mobile, " كود التفعيل هوا : $user->verification_code");
        $data = new UserResource($user);
        return response()->json(['value' => true, 'data' => $data, 'msg' => 'تم اعاده ارسال كود التفعيل', 'code' => 200]);


    }

    public function update_profile(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => 'التوكن خطأ', 'code' => 401]);
        }
        if ($user->type == 'poet') {
            $note = 'required';
            $type = 'required|in:dialogue,elite,all';
        } elseif ($user->type == 'class_manager') {
            $note = 'required';
        } else {
            $note = 'nullable';
            $type = 'nullable';
        }
        $validator = \Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'unique:users,email,' . $user->id,
            'mobile' => 'unique:users,mobile,' . $user->id,
            'image' => 'nullable|image',
            'city_id' => 'required|exists:cities,id',
            'poet_types' => $type,
            'note' => $note
        ]);
        if ($validator->passes()) {
            $user['name'] = $request['name'];
            $user['email'] = $request['email'];
            $user['mobile'] = $request['mobile'];
            if ($request->hasFile('image')) {
                $user->setImageAttribute();
            }
            if ($user->type == 'poet') {
                $user['poet_types'] = $request['poet_types'];
                $user['note'] = $request['note'];
            } elseif ($user->type == 'class_manager') {
                $user['note'] = $request['note'];
            }
            $user->update();
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

}
