<?php

namespace App\Http\Resources;

use Alkoumi\LaravelHijriDate\Hijri;
use App\Models\Order;
use App\Models\Setting;
use Carbon\Carbon;
use Illuminate\Http\Resources\Json\JsonResource;

class OrderResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function toArray($request)
    {

        $setting = Setting::first();

        if ($this->user->type == 'poet') {
            $user_ratio = $setting && $setting->commission_poet ? $setting->commission_poet : 0;
        } else {
            $user_ratio = $setting && $setting->commission_manager ? $setting->commission_manager : 0;
        }

        $status = ['waiting' => 'تحت المراجعه', 'accept' => 'تم الموافقة', 'in_progress' => 'حالى', 'finished' => 'منتهى', 'cancel' => 'ملغي'];
        $poet_types = ['all' => 'شعراء الصفوة و شعراء المحاورة', 'dialogue' => 'شعراء المحاورة', 'elite' => 'شعراء الصفوة'];

        return [
            'id' => $this->id,
            'client_id' => $this->client_id,
            'client_name' => $this->client->name,
            'client_mobile' => $this->client->mobile,
            'client_ratio' => $setting && $setting->commission_client ? (integer)$setting->commission_client : 0,
            'user_id' => $this->user_id,
            'user_name' => $this->user->name,
            'user_mobile' => $this->user->mobile,
            'user_ratio' => $user_ratio,
            'status' => $this->type,
            'status_name' => $status[$this->type],
            'city_id' => $this->city_id,
            'city_name' => $this->city->name,
            'date' => $this->date ? Hijri::FullDateIndicDigits($this->date) : 'لم يحدد بعد',
            'price' => $this->price ? $this->price : 'لم يحدد بعد',
            'poet_types' => $this->user->poet_types ? $poet_types[$this->user->poet_types] : '',
        ];
    }
}
