<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;

Route::get('/', function () {
    return redirect()->route('admin.login.view');
});

Route::get('admin/login', [HomeController::class, 'login'])->name('admin.login.view');
Route::post('admin/login', [HomeController::class, 'authenticate'])->name('admin.login.post');


Route::group(['middleware' => 'admin:admin', 'prefix' => 'admin'], function () {
    Route::get('/', [HomeController::class, 'index'])->name('admin');

    Route::get('logout', [HomeController::class, 'admin_logout'])->name('admin.logout');
    Route::get('profile', [HomeController::class, 'admin_profile'])->name('admin.profile');
    Route::patch('profile/update', [HomeController::class, 'update_own_profile'])->name('update_own_profile');

    Route::get('deposits/{id}/status/{status}', [DepositController::class, 'change_status'])->name('deposit.change');

    Route::resources([
        'ads' => AdsController::class,
        'settings' => SettingsController::class,
        'admins' => AdminsController::class,
        'cities' => CityController::class,
        'deposits' => DepositController::class,
        'banks' => BanksController::class,
        'orders' => OrderController::class,
        'transfers' => TransferController::class,
        'contactMessages' => ContactMessagesController::class,
    ]);
    Route::put('orders/changeStatus/{id}', [OrderController::class, 'changeStatus'])->name('orders.changeStatus');
    Route::put('orders/transfers/{id}', [TransferController::class, 'changeStatus'])->name('transfers.changeStatus');

    Route::delete('users/delete/{id}', [UsersController::class, 'destroy'])->name('users.destroy');
    Route::get('users/edit/{id}', [UsersController::class, 'edit'])->name('users.edit');
    Route::put('users/{id}', [UsersController::class, 'update'])->name('users.update');
    Route::get('users/{type}', [UsersController::class, 'index'])->name('users.index');
    Route::get('users/create/{type}', [UsersController::class, 'create'])->name('users.create');
    Route::post('users', [UsersController::class, 'store'])->name('users.store');
    Route::get('users/{id}/orders', [UsersController::class, 'orders'])->name('users.orders');


});
